
/* 
 * GetDataSize:
 * Calculate the position and data size of a specified region for DEM, 
 * LULC or Landsat data and return the result to the server program.
 *
 * ReturnData:
 * Read a specified section of the DEM, LULC or Landsat data from the
 * appropriate file and return the data to the server program.
*/

#include <stdio.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

/* Data types */
#define DEM          1
#define LANDSAT      2
#define LULC         3
#define DEM_LANDSAT  4
#define DEM_LULC     5
#define IMAGE3D      6
 
/* Error types */
#define CONNECT_ERROR  1
#define DATA_ERROR     2
#define RANGE_ERROR    3

#define DEM_bytes    2    /* short int */
#define Image_bytes  3    /* 1 char each for RGB */
#define Header_bytes 24   /* ints for N, S, E, W, rows, columns */

/* Grid spacings in meters for the different data */
#define DEMGridSpacing     100
#define LULCGridSpacing    100
#define LandsatGridSpacing  30

/* Coordinates of subset of data to be output */
int x0, x1, y0, y1;

main()
{
int DataType, StartRow, StartColumn, Rows, Columns, Scale;
void *data;
int errormsg, datasize;

   datasize = InputData(&DataType, &StartRow, &StartColumn, &Rows, &Columns, &Scale);
printf("%d %d %d %d %d %d \n", 
        DataType, StartRow, StartColumn, Rows, Columns, Scale);
printf("datasize = %d bytes\n", datasize);
   if ((data = malloc(datasize)) == NULL) {
     printf("can't allocate memory for data!\n");
   }

   errormsg = ReturnData(DataType, StartRow, StartColumn, Rows, Columns, Scale, data);
   if (errormsg != 0) {
     printf("Error %d\n", errormsg);
   }

   OutputData(data, datasize);

   exit(0);
}

int
InputData(pDataType, pStartRow, pStartColumn, pRows, pColumns, pScale)
int *pDataType, *pStartRow, *pStartColumn, *pRows, *pColumns, *pScale;
{
   int bufsize, datasize;

   scanf("%d %d %d %d %d %d \n", 
	 pDataType, pStartRow, pStartColumn, pRows, pColumns, pScale);

switch(*pDataType) {
  case DEM:
     bufsize = DEM_bytes;
     break;
  case LULC:
     bufsize = Image_bytes;
     break;
  case LANDSAT:
     bufsize = Image_bytes;
     break;
  case DEM_LULC:
     bufsize = DEM_bytes + Image_bytes;
     break;
  case DEM_LANDSAT:
     bufsize = DEM_bytes + Image_bytes;
     break;
  case IMAGE3D:
     return 0;
     break;
  default:
     return 0;
     break;
}
  datasize = *pRows * *pColumns * bufsize;
  return (datasize);
}


int
OutputData(data, datasize)
void *data;
int datasize;
{
  int OutputFile;
  char Header[256];

  /* PPM header file */
  if ((OutputFile = open("header", O_WRONLY | O_CREAT | O_TRUNC,
			  S_IRUSR |  S_IWUSR | S_IRGRP)) == -1)
  {
    printf("Error -- cannot open output header\n");
    exit(1);
  }
  sprintf(Header, "P6\n%d %d\n255\n\0", y1-y0,  x1-x0);
  write(OutputFile, Header, strlen(Header));
  close(OutputFile);

  /* RGB data file */
  if ((OutputFile = open("datafile", O_WRONLY | O_CREAT | O_TRUNC,
			  S_IRUSR |  S_IWUSR | S_IRGRP)) == -1)
  {
    printf("Error -- cannot open output datafile\n");
    exit(1);
  }
  write(OutputFile, data, datasize);
  close(OutputFile);
}


/* /////////////////////////////////////////////////////////////////// */

/* -----------------------------------------------------------------------
Name: int ReturnData(Args *ptr)
In: int DataType, StartRow, StartColumn, Rows, Columns, Scale;
Out: void *data;
Comments: This function returns the map segment data size
---------------------------------------------------------------------- */

int 
ReturnData(DataType, StartRow, StartColumn, Rows, Columns, Scale, data)
int DataType, StartRow, StartColumn, Rows, Columns, Scale;
void *data;
{
   int    InputDEM;
   int    InputLULC;
   int    InputLandsat;
   char   filename[256];
   int    North = 0;
   int    South = 0;
   int    East = 0;
   int    West = 0;
   int    M = 0;
   int    N = 0;
   int    errmsg = 0;

   /* Set up the data coordinates */
   x0 = StartRow;
   x1 = StartRow + Rows;
   y0 = StartColumn;
   y1 = StartColumn + Columns;

printf("x0 %d x1 %d y0 %d y1 %d \n", x0, x1, y0, y1);

switch(DataType) {

  case DEM:
     /* Open DEM data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/100m/albany_dem100.data");
     if ((InputDEM = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     ReadHeader(InputDEM, &North, &South, &East, &West, &M, &N);
     ReadData(InputDEM, x0, x1, y0, y1, M, N, DEM_bytes, data);
     break;

  case LULC:
     /* Open LULC data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/100m/albany_lulc100.data");
     if ((InputLULC = open(filename, O_RDONLY)) == -1) {
printf("Failed to open file %s\n", filename);
       return DATA_ERROR;
     }
     ReadHeader(InputLULC, &North, &South, &East, &West, &M, &N);
printf("N %d S %d E %d W %d \n", North, South, East, West);
printf("M %d N %d\n", M, N);
     ReadData(InputLULC, x0, x1, y0, y1, M, N, Image_bytes, data);
     break;

  case LANDSAT:
     /* Open 30m Landsat data file */
/*     sprintf(filename, "/projects/kyk02/terrain/NYS/30m/albany_landsat.data"); */
     sprintf(filename, "/project/kyk02/terrain/data/NYS/datafiles/landsat.albany"); 
     if ((InputLandsat = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     ReadHeader(InputLandsat, &North, &South, &East, &West, &M, &N);
     ReadData(InputLandsat, x0, x1, y0, y1, M, N, Image_bytes, data);
     break;

  case DEM_LULC:
     /* Open DEM data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/100m/albany_dem100.data");
     if ((InputDEM = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     /* Open LULC data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/100m/albany_lulc100.data");
     if ((InputLULC = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     /*
     ReadHeader(InputDEM, &North, &South, &East, &West, &M, &N);
     ReadHeader(InputLULC, &North, &South, &East, &West, &M, &N);
     ReadData2(InputDEM, InputLULC, x0, x1, y0, y1, M, N, 
	       DEM_bytes, Image_bytes, data);
     */
     return DATA_ERROR;
     break;

  case DEM_LANDSAT:
     /* Open 30m DEM data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/30m/albany_dem30.data");
     if ((InputDEM = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     /* Open 30m Landsat data file */
     sprintf(filename, "/projects/kyk02/terrain/NYS/30m/albany_landsat.data");
     if ((InputLandsat = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     /*
     ReadHeader(InputDEM, &North, &South, &East, &West, &M, &N);
     ReadHeader(InputLandsat, &North, &South, &East, &West, &M, &N);
     ReadData2(InputDEM, InputLandsat, x0, x1, y0, y1, M, N, 
	       DEM_bytes, Image_bytes, data);
     */
     return DATA_ERROR;
     break;

  case IMAGE3D:
     return DATA_ERROR;
     break;

  default:
     return DATA_ERROR;
     break;

}

/* NEED TO Fix up bogus data requests */
/*
   if ((x0 > M) || (x1 > M)) {
   }
   if ((y0 > N) || (y1 > N)) {
   }
*/

   return (errmsg);
}


/* Read in the coordinates for the binary data */
ReadHeader(fileptr, pNorth, pSouth, pEast, pWest, pM, pN)
int fileptr;
int *pNorth, *pSouth, *pEast, *pWest, *pM, *pN;
{
   read(fileptr, pNorth, sizeof(int));
   read(fileptr, pSouth, sizeof(int));
   read(fileptr, pEast,  sizeof(int));
   read(fileptr, pWest,  sizeof(int));
   read(fileptr, pM, sizeof(int));
   read(fileptr, pN, sizeof(int));
}


/* NEED TO ADD SCALE */

/* Read in the binary data */
ReadData(fp, x0, x1, y0, y1, M, N, bufsize, data)
int *fp;
int x0, x1, y0, y1, M, N;
int bufsize;
void *data;
{
   int x, linesize, FirstRow, LastRow, nread;
   long offset;

   linesize = bufsize * (y1 - y0); 

   /*  Data is ordered from top left to bottom right, so need to    *
    *  count down from the largest value of i (i.e. North to South) */
   FirstRow = N - x1;
   LastRow  = N - x0;

printf("linesize = %d  FirstRow = %d  LastRow = %d\n", 
       linesize, FirstRow, LastRow);
   
   for (x = FirstRow; x < LastRow; x++) 
   {
      /* Jump to the right section of the data file */
      offset = bufsize * (x * M + y0) + Header_bytes;
      lseek(fp, offset, 0);

      /* Read in rows of binary data */
      nread = read(fp, data, linesize);
      data += nread;

   }
   close(fp);
}



/* -----------------------------------------------------------------------
Name: int *GetDataSize(Args *ptr)
In: int DataType; 
    double dSouth, dWest, dNorth, dEast;
Out: int *StartRow, *StartColumn, *Rows, *Columns;
Comments: This function returns the map segment data size
---------------------------------------------------------------------- */
int 
GetDataSize(DataType, dSouth, dWest, dNorth, dEast, 
	    pStartRow, pStartColumn, pRows, pColumns)
int DataType; 
double dSouth, dWest, dNorth, dEast;
int *pStartRow, *pStartColumn, *pRows, *pColumns;
{ 
  int   InputFile;
  char  filename[256];
  int   North,  South,  East,  West;
  int   NorthEdge,  SouthEdge,  EastEdge,  WestEdge;
  int   M, N;
  int   GridSpacing, bufsize;
  static int    datasize = 0;

/*
  sscanf(ptr->coord_str, "%d %lf %lf %lf %lf", 
	 &DataType, &dSouth, &dWest, &dNorth, &dEast);
*/

  /* NEED TO Convert lat/long to UTM */
  /*
  LatLong_To_UTM(dSouth, dWest, &South, &West);
  LatLong_To_UTM(dNorth, dEast, &North, &East);
  */

switch(DataType) {

  case DEM:
     GridSpacing = DEMGridSpacing;
     bufsize = DEM_bytes;
     /* Open 100m DEM data file */
     sprintf(filename, "100m/albany_dem100.data");
     if ((InputFile = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     break;

  case LULC:
     GridSpacing = LULCGridSpacing;
     bufsize = Image_bytes;
     /* Open 100m LULC data file */
     sprintf(filename, "100m/albany_lulc100.data");
     if ((InputFile = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     break;

  case LANDSAT:
     GridSpacing = LandsatGridSpacing;
     bufsize = Image_bytes;
     /* Open 30m Landsat data file */
     sprintf(filename, "30m/albany_landsat.data");
     if ((InputFile = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     break;

  case DEM_LULC:
     GridSpacing = DEMGridSpacing;
     bufsize = DEM_bytes + Image_bytes;
     /* Open 100m DEM data file */
     sprintf(filename, "100m/albany_dem100.data");
     if ((InputFile = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     break;

  case DEM_LANDSAT:
     GridSpacing = LandsatGridSpacing;
     bufsize = DEM_bytes + Image_bytes;
     /* Open 30m Landsat data file */
     sprintf(filename, "30m/albany_landsat.data");
     if ((InputFile = open(filename, O_RDONLY)) == -1) {
       return DATA_ERROR;
     }
     break;

  case IMAGE3D:
     GridSpacing = LandsatGridSpacing;
     return DATA_ERROR;
     break;

}
  
  /* Read in the coordinates for the binary DEM data */
  read(InputFile, &NorthEdge, sizeof(int));
  read(InputFile, &SouthEdge, sizeof(int));
  read(InputFile, &EastEdge,  sizeof(int));
  read(InputFile, &WestEdge,  sizeof(int));
  read(InputFile, &M, sizeof(int));
  read(InputFile, &N, sizeof(int));  
  close (InputFile);

  *pStartRow = (South - SouthEdge) / GridSpacing;
  *pRows = (North - South) / GridSpacing;

  *pStartColumn = (West - WestEdge) / GridSpacing;
  *pColumns = (East - West) / GridSpacing;

  datasize = *pRows * *pColumns * bufsize;

  return (datasize);

}


